// $Id: imagefield_focus.js,v 1.8.2.1 2011/01/28 13:18:21 anrikun Exp $

/**
 * @file
 * Written by Henri MEDOT <henri.medot[AT]absyx[DOT]fr>
 * http://www.absyx.fr
 */

Drupal.behaviors.imagefield_focus = function(context) {

  // Helper functions.
  var valToSel = function(val) {
    var sel = val.split(',');
    if (sel.length != 4) {
      return null;
    }

    for (var i = 0; i < 4; i++) {
      var v = Number(sel[i]);
      if (isNaN(v)) {
        return null;
      }
      sel[i] = Math.round(v);
    }

    if ((sel[0] < 0) || (sel[1] < 0) || (sel[2] <= 0) || (sel[3] <= 0)) {
      return null;
    }

    return sel;
  };

  var selToVal = function(sel) {
    return (sel) ? sel.join(',') : '';
  };

  // Behavior.
  window.setTimeout(function() {
    $('.imagefield-focus.focus-box:not(.imagefield-focus-processed)', context).addClass('imagefield-focus-processed').each(function() {
      var $this = $(this);
      var img = $('> img', $this);
      var inputs = $('input.imagefield-focus', $this.parent());
      var fieldName = inputs.attr('name').replace(/^([a-z0-9_]+)\[.+$/, '$1');
      var settings = Drupal.settings.imagefield_focus[fieldName];

      var minSize, ratio;
      if (settings.min_width && settings.min_height) {
        minSize = [settings.min_width, settings.min_height];
        ratio = settings.lock_ratio;
      }

      var options = {
        boxSize: $this.width() || parseInt($this.css('min-width')),
        realSize: img.attr('alt').split('x'),
        minSize: minSize
      };
      img.css('display', 'block').imgfocus(options);

      inputs.focus(function(e, init) {
        img.unbind('change.imagefield-focus');
        var input = $(this).addClass('active');
        var other = inputs.eq((inputs.index(input) + 1) % 2).removeClass('active');
        var otherSel = valToSel(other.val());
        var options = (input.hasClass('crop-rect')) ? {
          ratio: 0,
          minRect: otherSel,
          maxRect: null
        } : {
          ratio: ratio,
          minRect: null,
          maxRect: otherSel
        };
        img.imgfocus('options', options).imgfocus('selection', valToSel(input.val()));
        img.bind('change.imagefield-focus', function() {
          input.val(selToVal(img.imgfocus('selection')));
        });

        if (init) {
          e.stopPropagation();
          e.preventDefault();
          return false;
        }
      }).change(function() {
        $(this).val(selToVal(img.imgfocus('selection', valToSel($(this).val())).imgfocus('selection')));
      });

      inputs.eq(0).trigger('focus', [true]);
    });
  }, 100);
};
