<?php
/**
 * @file
 * p6_profile.profile
 */

// Site name.
define('SITE_NAME', 'Project6');

// Site E-Mail.
define('SITE_MAIL', '');

// Administrator username.
define('ADMINISTRATOR_USERNAME', 'superadmin');

// Administrator password.
// If NULL, a random alphanumeric password will be generated.
define('ADMINISTRATOR_PASSWORD', NULL);

// Administrator e-mail.
define('ADMINISTRATOR_EMAIL', '');

// Default theme.
define('DEFAULT_THEME', 'p6');

// Name of "admin" role. will get rid 3.
define('PROJECT6_EDITOR_NAME', 'editor');

// Name of "admin" role. will get rid 4.
define('PROJECT6_ADMIN_NAME', 'admin');

/**
 * Return a description of the profile for the initial installation screen.
 *
 * @return array
 *   An array with keys 'name' and 'description' describing this profile,
 *   and optional 'language' to override the language selection for
 *   language-specific profiles.
 */
function p6_profile_profile_details() {
  return array(
    'name' => 'Project6',
    'description' => 'Drupal distribution by Project6 Design, Inc.',
  );
}

/**
 * Return an array of the modules to be enabled when this profile is installed.
 *
 * @return array
 *   An array of modules to enable.
 */
function p6_profile_profile_modules() {
  $core_modules = array(
    // Required core modules.
    'block',
    'filter',
    'node',
    'system',
    'user',
    // Optional core modules.
    'dblog',
    'help',
    'menu',
    'path',
    'search',
    'update',
  );

  $contributed_modules = array(
    'install_profile_api',
    'token',

    // Views.
    'views',
    'views_export',
    'views_ui',
    'views_ui_basic',

    // CCK.
    'content',
    'content_copy',
    'fieldgroup',
    'filefield',
    'imagefield',
    'optionwidgets',
    'number',
    'text',
    'nodereference',
    'userreference',
    'autocomplete_widgets',
    'link',
    'imagefield_focus',
    'filefield_paths',
    'filefield_sources',
    'filefield_stats',
    'viewfield',

    'draggableviews_cck',
    'draggableviews',


    'libraries',

    'jquery_ui',
    'jquery_update',
    'colorbox',

    // Date.
    'date_api',
    'date',
    'date_popup',
    'date_timezone',

    // Imagecache.
    'imageapi',
    'imageapi_gd',
    'imagecache',
    'imagecache_ui',
    'smartcrop',

    'auto_nodetitle',
    'ctm',
    'automenu',
    'automenu_node',
    'menu_block',

    // Development.
    'backup_migrate',
    'db_maintenance',
    'demo',
    'devel',
    'diff',
    'googleanalytics',

    // IMCE.
    'imce',
    'imce_mkdir',
    'imce_wysiwyg',

    // Linkit.
    'linkit',
    'linkit_node',
    'linkit_views',

    'admin',
    'admin_tools',

    'nodeasblock',

    'node_embed',
    'wysiwyg',
    'vertical_tabs',

    // Input filters.
    'better_formats',
    'pathfilter',
    'image_resize_filter',
    'wysiwyg_filter',

    'transliteration',
    'page_title',
    'globalredirect',
    'pathauto',
    'path_redirect',

    'node_tools',
    'revisioning',
    'revisioning_scheduler',

    'semiclean',
    'poormanscron',

    'p6_helper',
  );

  return array_merge($core_modules, $contributed_modules);
}

/**
 * Return a list of tasks that this profile supports.
 *
 * @return array
 *   A keyed array of tasks the profile will perform during
 *   the final stage. The keys of the array will be used internally,
 *   while the values will be displayed to the user in the installer
 *   task list.
 */
function p6_profile_profile_task_list() {
  global $conf;
  $conf['site_name'] = 'Project6';
  $conf['theme_settings'] = array(
    'default_logo' => 0,
    'logo_path' => 'profiles/p6_profile/logo.png',
  );

  $tasks['p6-configure-batch'] = st('Configure Project6 installation');
  return $tasks;
}

/**
 * Perform any final installation tasks for this profile.
 *
 * @param string $task
 *   The current $task of the install system. When hook_profile_tasks()
 *   is first called, this is 'profile'.
 * @param string $url
 *   Complete URL to be used for a link or form action on a custom page,
 *   if providing any, to allow the user to proceed with the installation.
 *
 * @return string
 *   An optional HTML string to display to the user. Only used if you
 *   modify the $task, otherwise discarded.
 */
function p6_profile_profile_tasks(&$task, $url) {
  $output = '';

  // Use the Install Profile API.
  install_include(p6_profile_profile_modules());

  if ($task == 'profile') {
    drupal_set_title(t('Project6 Installation'));
    _p6_profile_log(t('Starting installation'));
    _p6_profile_core_settings();
    $task = 'p6-configure';
  }

  if ($task == 'p6-configure') {
    $batch['title'] = st('Configuring @drupal', array('@drupal' => drupal_install_profile_name()));

    $batch['operations'][] = array('_p6_profile_set_roles', array());
    $batch['operations'][] = array('_p6_profile_set_users', array());
    $batch['operations'][] = array('_p6_profile_set_permissions', array());
    $batch['operations'][] = array('_p6_profile_set_contenttypes', array());
    $batch['operations'][] = array('_p6_profile_set_menus', array());
    $batch['operations'][] = array('_p6_profile_set_blocks', array());
    $batch['operations'][] = array('_p6_profile_set_views', array());
    $batch['operations'][] = array('_p6_profile_set_wysiwyg', array());
    $batch['operations'][] = array('_p6_profile_set_inputfilters', array());
    $batch['operations'][] = array('_p6_profile_set_variables', array());
    $batch['operations'][] = array('_p6_profile_set_backupmigrate', array());
    $batch['operations'][] = array('_p6_profile_cleanup', array());

    $batch['error_message'] = st('There was an error configuring @drupal.', array('@drupal' => drupal_install_profile_name()));
    $batch['finished'] = '_p6_profile_configure_finished';

    variable_set('install_task', 'p6-configure-batch');

    batch_set($batch);
    batch_process($url, $url);
  }

  // Land here until the batches are done.
  if ($task == 'p6-configure-batch') {
    include_once 'includes/batch.inc';
    $output = _batch_page();
  }

  return $output;
}

/**
 * Installation tasks are finished.
 */
function _p6_profile_configure_finished($success, $results) {
  _p6_profile_log(t('Project6 has been installed.'));
  variable_set('install_task', 'profile-finished');
}

/**
 * Installation task to set up core settings.
 */
function _p6_profile_core_settings() {
  install_default_theme(DEFAULT_THEME);
  install_disable_theme('garland');
  variable_set('maintenance_theme', DEFAULT_THEME);

  // Anonymous User.
  variable_set('anonymous', 'Visitor');

  // Error logging.
  variable_set('error_level', '0');

  // File System.
  variable_set('file_directory_path', 'sites/default/files');
  variable_set('file_downloads', 1);

  // User settings.
  variable_set('user_register', 0);
  variable_set('user_pictures', 0);
  variable_set('user_picture_path', 'avatars');
  variable_set('user_picture_default', '');
  variable_set('user_picture_dimensions', '80x80');
  variable_set('user_picture_file_size', '1024');
  variable_set('user_picture_guidelines', '');

  // Available Updates.
  variable_set('update_notify_emails', ADMINISTRATOR_EMAIL);
  variable_set('update_notification_threshold', 'security');
  variable_set('update_check_frequency', '1');

  // Date - America/Los_Angeles
  variable_set('date_default_timezone', -25200);
  variable_set('date_default_timezone_name', 'America/Los_Angeles');
  variable_set('date_format_long', 'l, F j, Y - g:ia');
  variable_set('date_format_medium', 'F j, Y - g:ia');
  variable_set('date_format_short', 'm/d/Y - g:ia');
  variable_set('configurable_timezones', '0');

  _p6_profile_log(st('Completed configuring core settings'));
}

/**
 * Installation task to set up content types.
 */
function _p6_profile_set_contenttypes() {
  $types = array(
    array(
      'type' => 'page',
      'name' => st('Basic page'),
      'module' => 'node',
      'description' => st("Use <em>basic pages</em> for your static content, such as an 'About us' page."),
      'custom' => FALSE,
      'modified' => FALSE,
      'locked' => TRUE,
      'help' => '<strong>Submission Guidelines</strong><br />Do not forget to update this guideline!',
      'min_word_count' => '',
    ),
    array(
      'type' => 'block',
      'name' => st('Custom block'),
      'module' => 'node',
      'description' => st("Use <em>Custom blocks</em> as Drupal's native block replacements."),
      'custom' => FALSE,
      'modified' => FALSE,
      'locked' => TRUE,
      'help' => '<strong>Submission Guidelines</strong><br />Do not forget to update this guideline!',
      'min_word_count' => '',
    ),
  );

  foreach ($types as $type) {
    $type = (object) _node_type_set_defaults($type);
    node_type_save($type);
  }

  // Content-type: Page.
  variable_set('ant_page', '0');
  variable_set('ant_pattern_page', '');
  variable_set('ant_php_page', 0);
  variable_set('comment_page', 0);
  variable_set('menu_page', array(
    0 => 'footer-links',
    1 => 'primary-links',
    2 => 'secondary-links',
  ));
  variable_set('page_menu_settings', array(
    'footer-links' => 'footer-links',
    'primary-links' => 'primary-links',
    'secondary-links' => 'secondary-links',
  ));
  variable_set('nodeasblock_page', 0);
  variable_set('node_options_page', array('revision', 'revision_moderation'));
  variable_set('page_title_type_page_showfield', 1);
  variable_set('new_revisions_page', 1);
  variable_set('revisioning_auto_publish_page', 1);
  variable_set('show_diff_inline_page', 0);
  variable_set('show_preview_changes_page', 1);
  variable_set('enable_revisions_page_page', 1);

  // Content-type: Block.
  $nodeasblock_region_block = array(
    0 => 'header',
    1 => 'header_bottom',
    2 => 'page_hero',
    3 => 'content_top',
    4 => 'content',
    5 => 'content_bottom',
    6 => 'sidebar_first',
    7 => 'sidebar_last',
    8 => 'footer',
    9 => 'footer_bottom',
  );
  variable_set('ant_block', '0');
  variable_set('ant_pattern_block', '');
  variable_set('ant_php_block', 0);
  variable_set('block_menu_settings', array());
  variable_set('comment_block', 0);
  variable_set('menu_block', array());
  variable_set('nodeasblock_block', 2);
  variable_set('nodeasblock_default_region_block', 'sidebar_last');
  variable_set('nodeasblock_default_weight_block', 0);
  variable_set('nodeasblock_region_block', $nodeasblock_region_block);
  variable_set('node_options_block', array('revision', 'revision_moderation'));
  variable_set('page_title_type_block', '');
  variable_set('page_title_type_block_showfield', 0);
  variable_set('new_revisions_block', 1);
  variable_set('revisioning_auto_publish_block', 1);
  variable_set('show_diff_inline_block', 0);
  variable_set('show_preview_changes_block', 1);
  variable_set('enable_revisions_page_block', 1);

  // Update theme settings.
  $theme_settings = variable_get('theme_settings', array());
  $theme_settings['toggle_node_info_block'] = FALSE;
  $theme_settings['toggle_node_info_page'] = FALSE;
  variable_set('theme_settings', $theme_settings);

  _p6_profile_log(st('Configured content-types.'));
}

/**
 * Installation task to set up roles.
 *
 * Adds 'admin' and 'editor' roles.
 * Adds User #1 to 'admin' role.
 */
function _p6_profile_set_roles() {
  // Editor should get rid of 3.
  install_add_role(PROJECT6_EDITOR_NAME);
  // Admin should get rid of 4.
  install_add_role(PROJECT6_ADMIN_NAME);
  // Add User #1 to admin role.
  install_add_user_to_role(1, install_get_rid(PROJECT6_ADMIN_NAME));

  _p6_profile_log(st('Configured roles.'));
}

/**
 * Installation task to set up users.
 */
function _p6_profile_set_users() {
  install_add_user('admin', 'admin', 'admin@localhost', $roles = array('admin'), $status = 1);
  install_add_user('editor', 'editor', 'editor@localhost', $roles = array('editor'), $status = 1);
}

/**
 * Installation task to set up permissions.
 *
 * Defines permissions for following roles:
 *  - anonymous user
 *  - authenticated user
 *  - editor
 *  - admin
 */
function _p6_profile_set_permissions() {
  $permissions_remove = array(
    'collapse format fieldset by default',
    'collapsible format selection',
    'show format selection for blocks',
    'show format selection for comments',
    'show format selection for nodes',
    'show format tips',
    'show more format tips link',
  );
  _p6_profile_remove_permissions('anonymous user', $permissions_remove);
  _p6_profile_remove_permissions('authenticated user', $permissions_remove);
  _p6_profile_remove_permissions(PROJECT6_EDITOR_NAME, $permissions_remove);
  _p6_profile_remove_permissions(PROJECT6_ADMIN_NAME, $permissions_remove);

  $permissions_anonymous_user = array(
    // Comment module.
    'access comments',
    'post comments',
    // Node module.
    'access content',
    // Search module.
    'search content',
  );

  $permissions_authenticated_user = array(
    // Better formats module.
    'show format tips',
    'show more format tips link',
    // Comment module.
    'access comments',
    'post comments',
    // Devel module.
    // 'access devel information',
    // 'switch users',
    'access content',
    // Gravatar module.
    'disable own gravatar',
    'use gravatar',
    // Search module.
    'search content',
  );

  $permissions_editor = array(
    // Admin module.
    'use admin toolbar',
    // Backup migrate module.
    'access backup and migrate',
    'perform backup',
    // Comment module.
    'post comments without approval',
    // Draggable views module.
    'Allow Reordering',
    // Menu module.
    'administer menu',
    // Node module.
    'create block content',
    'create page content',
    'delete own block content',
    'delete own page content',
    'edit any block content',
    'edit any page content',
    'edit own block content',
    'edit own page content',
    'revert revisions',
    'view revisions',
    // Nodeasblock module.
    'create blocks for nodes',
    // Page title module.
    'set page title',
    // Path module.
    'create url aliases',
    // Revisioning module.
    'edit revisions',
    'view revision status messages',
    'view revisions of any block content',
    'view revisions of any page content',
    // System module.
    'access administration pages',
    // Taxonomy module.
    'administer taxonomy',
    // Views UI Basic module.
    'edit views basic settings',
  );

  $permissions_admin = array(
    // Admin module.
    'use admin toolbar',
    // Backup migrate module.
    'access backup and migrate',
    'access backup files',
    'perform backup',
    'restore from backup',
    // Block module.
    'administer blocks',
    // Comment module.
    'administer comments',
    'post comments without approval',
    // Draggable views module.
    'Allow Reordering',
    // Filefield stats module.
    'view filefield stats',
    // Google analytics module.
    'administer google analytics',
    // Gravater module.
    'administer gravatar',
    // Imageapi module.
    'administer imageapi',
    // Imagecache module.
    'administer imagecache',
    'flush imagecache',
    // Menu module.
    'administer menu',
    // Node module.
    'create block content',
    'create page content',
    'delete any block content',
    'delete any page content',
    'delete own block content',
    'delete own page content',
    'delete revisions',
    'edit any block content',
    'edit any page content',
    'edit own block content',
    'edit own page content',
    'revert revisions',
    'view revisions',
    // Nodeasblock module.
    'administer blocks for nodes',
    'create blocks for nodes',
    // Page title module.
    'administer page titles',
    'set page title',
    // Path module.
    'administer url aliases',
    'create url aliases',
    // Path redirect module.
    'administer redirects',
    // Revisioning module.
    'edit revisions',
    'publish revisions',
    'unpublish current revision',
    'view revision status messages',
    'view revisions of any block content',
    'view revisions of any page content',
    // System module.
    'access administration pages',
    'access site reports',
    'administer files',
    'administer site configuration',
    // Taxonomy module.
    'administer taxonomy',
    // Views UI Basic module.
    'edit views basic settings',
  );

  _p6_profile_add_permissions('anonymous user', $permissions_anonymous_user);
  _p6_profile_add_permissions('authenticated user', $permissions_authenticated_user);
  _p6_profile_add_permissions(PROJECT6_EDITOR_NAME, $permissions_editor);
  _p6_profile_add_permissions(PROJECT6_ADMIN_NAME, $permissions_admin);

  _p6_profile_log(st('Configured permissions.'));
}

/**
 * Callback function to set permissions.
 *
 * @param string $name
 *   Role name.
 *
 * @param array $permissions
 *   Permissions.
 */
function _p6_profile_add_permissions($name, $permissions = array()) {
  // Get the role ID for the role name.
  $rid = install_get_rid($name);
  // Add the permissions for this role.
  if ($rid) {
    install_add_permissions($rid, $permissions);
  }
}

/**
 * Callback function to remove permissions.
 *
 * @param string $name
 *   Role name.
 *
 * @param array $permissions
 *   Permissions.
 */
function _p6_profile_remove_permissions($name, $permissions = array()) {
  // Get the role ID for the role name.
  $rid = install_get_rid($name);
  // Remove the permissions for this role.
  if ($rid) {
    install_remove_permissions($rid, $permissions);
  }
}

/**
 * Installation task to set up menus.
 *
 * Updates Primary and Secondary menu descriptions.
 */
function _p6_profile_set_menus() {
  $new_primary_links_desc = 'Primary links are often used at the theme layer to show the major sections of a site.';
  $new_secondary_links_desc = 'Secondary links are often used as a meta navigation for pages like contact details, FAQ and other secondary navigation items.';
  db_query("UPDATE {menu_custom} SET description = '%s' WHERE menu_name = '%s'", $new_primary_links_desc, 'primary-links');
  db_query("UPDATE {menu_custom} SET description = '%s' WHERE menu_name = '%s'", $new_secondary_links_desc, 'secondary-links');

  // Update the menu router information.
  menu_rebuild();

  _p6_profile_log(st('Configured menus.'));
}

/**
 * Installation task to set up blocks.
 */
function _p6_profile_set_blocks() {
  install_init_blocks();

  db_query("UPDATE {blocks} SET region = '', status = 0 WHERE theme = '%s' AND module != 'p6_helper'", DEFAULT_THEME);

  install_set_block('p6_helper', 'footer', DEFAULT_THEME, 'footer');
  install_set_block('p6_helper', 'logo', DEFAULT_THEME, 'footer_bottom');
}

/**
 * Installation task to set up views.
 */
function _p6_profile_set_views() {
  // content-summary view is enabled by default, disable it.
  $view = views_get_view('revisioning_content_summary');
  $view->disabled = TRUE;
  $view->save();

  // Import views.
  $view_files = file_scan_directory(dirname(__FILE__) . '/views', '.*\.php$');
  foreach ($view_files as $file) {
    install_views_ui_import_from_file($file->filename);
  }
}

/**
 * Installation task to set up WYSIWYG profiles.
 */
function _p6_profile_set_wysiwyg() {
  $wysiwyg_profiles = array();
  $wysiwyg_profiles['Full HTML'] = 'a:20:{s:7:"default";i:1;s:11:"user_choose";i:0;s:11:"show_toggle";i:1;s:5:"theme";s:8:"advanced";s:8:"language";s:2:"en";s:7:"buttons";a:12:{s:7:"default";a:27:{s:4:"bold";i:1;s:6:"italic";i:1;s:9:"underline";i:1;s:13:"strikethrough";i:1;s:11:"justifyleft";i:1;s:13:"justifycenter";i:1;s:12:"justifyright";i:1;s:11:"justifyfull";i:1;s:7:"bullist";i:1;s:7:"numlist";i:1;s:7:"outdent";i:1;s:6:"indent";i:1;s:4:"undo";i:1;s:4:"redo";i:1;s:6:"unlink";i:1;s:6:"anchor";i:1;s:5:"image";i:1;s:7:"cleanup";i:1;s:3:"sup";i:1;s:3:"sub";i:1;s:10:"blockquote";i:1;s:3:"cut";i:1;s:4:"copy";i:1;s:5:"paste";i:1;s:9:"visualaid";i:1;s:12:"removeformat";i:1;s:7:"charmap";i:1;}s:8:"advimage";a:1:{s:8:"advimage";i:1;}s:11:"contextmenu";a:1:{s:11:"contextmenu";i:1;}s:4:"font";a:2:{s:12:"formatselect";i:1;s:11:"styleselect";i:1;}s:10:"fullscreen";a:1:{s:10:"fullscreen";i:1;}s:5:"paste";a:2:{s:9:"pastetext";i:1;s:9:"pasteword";i:1;}s:5:"table";a:1:{s:13:"tablecontrols";i:1;}s:5:"media";a:1:{s:5:"media";i:1;}s:10:"xhtmlxtras";a:3:{s:4:"cite";i:1;s:4:"abbr";i:1;s:7:"acronym";i:1;}s:4:"imce";a:1:{s:4:"imce";i:1;}s:6:"linkit";a:1:{s:6:"linkit";i:1;}s:6:"drupal";a:1:{s:10:"node_embed";i:1;}}s:11:"toolbar_loc";s:3:"top";s:13:"toolbar_align";s:4:"left";s:8:"path_loc";s:6:"bottom";s:8:"resizing";i:1;s:11:"verify_html";i:1;s:12:"preformatted";i:0;s:22:"convert_fonts_to_spans";i:1;s:17:"remove_linebreaks";i:1;s:23:"apply_source_formatting";i:1;s:27:"paste_auto_cleanup_on_paste";i:1;s:13:"block_formats";s:32:"p,address,pre,h2,h3,h4,h5,h6,div";s:11:"css_setting";s:5:"theme";s:8:"css_path";s:0:"";s:11:"css_classes";s:0:"";}';
  $wysiwyg_profiles['Filtered HTML'] = 'a:20:{s:7:"default";i:1;s:11:"user_choose";i:0;s:11:"show_toggle";i:0;s:5:"theme";s:8:"advanced";s:8:"language";s:2:"en";s:7:"buttons";a:5:{s:7:"default";a:9:{s:4:"bold";i:1;s:6:"italic";i:1;s:9:"underline";i:1;s:13:"strikethrough";i:1;s:7:"bullist";i:1;s:7:"numlist";i:1;s:6:"unlink";i:1;s:6:"anchor";i:1;s:5:"image";i:1;}s:8:"advimage";a:1:{s:8:"advimage";i:1;}s:5:"paste";a:2:{s:9:"pastetext";i:1;s:9:"pasteword";i:1;}s:4:"imce";a:1:{s:4:"imce";i:1;}s:6:"linkit";a:1:{s:6:"linkit";i:1;}}s:11:"toolbar_loc";s:3:"top";s:13:"toolbar_align";s:4:"left";s:8:"path_loc";s:6:"bottom";s:8:"resizing";i:1;s:11:"verify_html";i:1;s:12:"preformatted";i:0;s:22:"convert_fonts_to_spans";i:1;s:17:"remove_linebreaks";i:1;s:23:"apply_source_formatting";i:1;s:27:"paste_auto_cleanup_on_paste";i:1;s:13:"block_formats";s:32:"p,address,pre,h2,h3,h4,h5,h6,div";s:11:"css_setting";s:5:"theme";s:8:"css_path";s:0:"";s:11:"css_classes";s:0:"";}';

  // $wysiwyg_profiles = _p6_profile_wysiwyg_profiles();
  foreach ($wysiwyg_profiles as $filter_name => $settings) {
    // $settings = serialize($settings);
    $query = "SELECT format FROM {filter_formats} WHERE name='%s'";
    $format = db_result(db_query($query, $filter_name));
    $query = "INSERT INTO {wysiwyg} (format, editor, settings) VALUES ('%d', '%s', '%s')";
    db_query($query, $format, 'tinymce', $settings);
  }

  _p6_profile_log(st('Configured WYSIWYG profiles.'));
}

/**
 * Installation task to set up input formats, filters and better formats.
 */
function _p6_profile_set_inputfilters() {
  // Add 'admin' and 'editor' roles to Full HTML input format.
  $roles = array();
  $roles[] = install_get_rid(PROJECT6_ADMIN_NAME);
  $roles[] = install_get_rid(PROJECT6_EDITOR_NAME);

  install_format_set_roles($roles, 2);

  $default_format = install_add_format('Plain text');

  // Values for some of the variable to be set.
  $wysiwyg_filter_nofollow_domains_1 = array(
    0 => 'example\.com',
  );

  $wysiwyg_filter_styles_text_1 = array(
    'text-align' => 'text-align',
    'text-decoration' => 'text-decoration',
  );

  $wysiwyg_filter_valid_elements_parsed_1 = array(
    'a' => array(
      'href' => array(
        'required' => TRUE,
      ),
      'target' => array(
        'values' => array(
          0 => '_blank',
        ),
      ),
      'title' => array(),
      'style' => array(),
    ),
    'br' => array(
      'style' => array(),
    ),
    'cite' => array(
      'style' => array(),
    ),
    'code' => array(
      'style' => array(),
    ),
    'dd' => array(
      'style' => array(),
    ),
    'dl' => array(
      'style' => array(),
    ),
    'dt' => array(
      'style' => array(),
    ),
    'em' => array(
      'style' => array(),
    ),
    'ul' => array(
      'style' => array(),
    ),
    'ol' => array(
      'style' => array(),
    ),
    'li' => array(
      'style' => array(),
    ),
    'span' => array(
      'style' => array(),
    ),
    'strong' => array(
      'style' => array(),
    ),
    'img' => array(
      'src' => array(
        'reguired' => TRUE,
      ),
      'width' => array(),
      'height' => array(),
      'style' => array(),
    ),
    'p' => array(
      'align' => array(
        'values' => array(
          0 => 'center',
          1 => 'justify',
          2 => 'left',
          3 => 'right',
        ),
      ),
      'style' => array(),
    ),
  );

  $wysiwyg_filter_valid_elements_raw_1 = "a[!href|target<_blank|title],\r\nimg[!src|width|height],\r\np[align<center?justify?left?right],\r\n@[style],\r\nbr,em,strong,span,cite,code,ul,ol,li,dl,dt,dd";

  $wysiwyg_filter_styles_font_1 = array(
    'font-style' => 'font-style',
  );

  $image_resize_filter_image_locations_2 = array(
    'local' => 'local',
    'remote' => 0,
  );

  $linkit_node = array(
    'display_settings' => array(
      'nid' => 'nid',
      'content_type' => 'content_type',
      'status' => 0,
      'language' => 0,
      'created' => 0,
      'changed' => 0,
      'show_unpublished' => 0,
    ),
    'content_types' => array('page' => 0),
  );

  // Filtered HTML settings.
  variable_set('allowed_html_1', '<a> <img> <em> <strong> <span> <cite> <code> <ul> <ol> <li> <dl> <dt> <dd>');
  variable_set('filter_html_1', '1');
  variable_set('filter_html_help_1', 1);
  variable_set('filter_html_nofollow_1', 0);
  variable_set('filter_url_length_1', '72');
  variable_set('wysiwyg_filter_allow_comments_1', '0');
  variable_set('wysiwyg_filter_nofollow_domains_1', $wysiwyg_filter_nofollow_domains_1);
  variable_set('wysiwyg_filter_nofollow_policy_1', 'whitelist');
  variable_set('wysiwyg_filter_styles_text_1', $wysiwyg_filter_styles_text_1);
  variable_set('wysiwyg_filter_valid_elements_parsed_1', $wysiwyg_filter_valid_elements_parsed_1);
  variable_set('wysiwyg_filter_valid_elements_raw_1', $wysiwyg_filter_valid_elements_raw_1);
  variable_set('wysiwyg_filter_styles_font_1', $wysiwyg_filter_styles_font_1);
  // Full HTML settings.
  variable_set('filter_url_length_2', '72');
  variable_set('image_resize_filter_link_rel_2', '');
  variable_set('image_resize_filter_link_2', 1);
  variable_set('image_resize_filter_link_class_2', 'colorbox-load');
  variable_set('image_resize_filter_image_locations_2', $image_resize_filter_image_locations_2);
  variable_set('pathfilter_link_type', 'relative');
  // Plain text settings.
  variable_set('allowed_html_' . $default_format, '');
  variable_set('filter_html_' . $default_format, '1');
  variable_set('filter_html_help_' . $default_format, 1);
  variable_set('filter_html_nofollow_' . $default_format, 0);
  variable_set('filter_url_length_' . $default_format, '72');

  // Linkit module settings.
  variable_set('linkit_node', $linkit_node);

  // Set filters in Input formats.
  // Filtered HTML.
  install_set_filter(1, 'pathfilter', 0, -10);
  install_set_filter(1, 'filter', 2, -9);
  install_set_filter(1, 'wysiwyg_filter', 0, -8);
  install_set_filter(1, 'filter', 1, -7);
  install_set_filter(1, 'filter', 3, -6);
  // Full HTML.
  install_set_filter(2, 'pathfilter', 0, -10);
  install_set_filter(2, 'filter', 2, -9);
  install_set_filter(2, 'filter', 1, -8);
  install_set_filter(2, 'image_resize_filter', 0, -7);
  install_set_filter(2, 'filter', 3, -6);
  install_set_filter(2, 'node_embed', 0, -5);
  // Plain text.
  install_set_filter($default_format, 'filter', 0, -9);
  install_set_filter($default_format, 'filter', 1, -8);
  install_set_filter($default_format, 'filter', 2, -10);

  // Default input format: Filtered HTML.
  variable_set('filter_default_format', $default_format);

  // Better Formats defaults.
  db_query("DELETE FROM {better_formats_defaults}");

  $rid_editor = install_get_rid(PROJECT6_EDITOR_NAME);
  $rid_admin  = install_get_rid(PROJECT6_ADMIN_NAME);

  $sql = "INSERT INTO {better_formats_defaults} (`rid`, `type`, `format`, `type_weight`, `weight`) VALUES (%d, '%s', %d, %d, %d)";

  // Admin.
  db_query($sql, $rid_admin, 'node', 2, 1, -25);
  db_query($sql, $rid_admin, 'comment', 2, 1, -25);
  db_query($sql, $rid_admin, 'block', 2, 1, -25);
  // Editor.
  db_query($sql, $rid_editor, 'node', 2, 1, -24);
  db_query($sql, $rid_editor, 'comment', 2, 1, -24);
  db_query($sql, $rid_editor, 'block', 2, 1, -24);
  // Authenticated user.
  db_query($sql, 2, 'node', 0, 1, 24);
  db_query($sql, 2, 'comment', 0, 1, 24);
  db_query($sql, 2, 'block', 0, 1, 24);
  // Anonymous user.
  db_query($sql, 1, 'node', 0, 1, 25);
  db_query($sql, 1, 'comment', 0, 1, 25);
  db_query($sql, 1, 'block', 0, 1, 25);

  _p6_profile_log(st('Configured Input formats and Filters.'));
}

/**
 * Installation task to set up remaining variables.
 */
function _p6_profile_set_variables() {
  // Transliteration.
  variable_set('transliteration_file_lowercase', 1);
  variable_set('transliteration_file_uploads', 1);

  // PathAuto.
  variable_set('pathauto_case', '1');
  variable_set('pathauto_ignore_words', '');
  variable_set('pathauto_reduce_ascii', 1);
  variable_set('pathauto_separator', '-');
  variable_set('pathauto_transliterate', 1);
  variable_set('pathauto_update_action', '3');
  variable_set('pathauto_verbose', 0);
  variable_set('pathauto_node_pattern', '[menupath-raw]');

  // Path Redirect.
  variable_set('path_redirect_allow_bypass', 0);
  variable_set('path_redirect_auto_redirect', 1);
  variable_set('path_redirect_default_status', '301');
  variable_set('path_redirect_purge_inactive', '0');
  variable_set('path_redirect_redirect_warning', 0);

  // Poorman's Cron - 1 Hour.
  variable_set('cron_safe_threshold', '3600');

  // Image API.
  variable_set('imageapi_image_toolkit', 'imageapi_gd');
  variable_set('imageapi_jpeg_quality', '100');
  variable_set('image_jpeg_quality', '100');
  variable_set('image_toolkit', 'gd');

  // Page Title.
  variable_set('page_title_default', '[page-title] | [site-name]');
  variable_set('page_title_front', '[site-name]');

  // Colorbox.
  variable_set('colorbox_compression_type', 'min');
  variable_set('colorbox_imagefield_gallery', '1');
  variable_set('colorbox_inline', 1);
  variable_set('colorbox_load', 1);

  // Superfish.
  variable_set('superfish_number', '1');

  // IMCE.
  $imce_profiles = array(
    1 => array(
      'name' => 'User-1',
      'usertab' => 1,
      'filesize' => '0',
      'quota' => '0',
      'tuquota' => '0',
      'extensions' => '*',
      'dimensions' => '1200x1200',
      'filenum' => '0',
      'directories' => array(
        0 => array(
          'name' => '.',
          'subnav' => 1,
          'browse' => 1,
          'upload' => 1,
          'thumb' => 1,
          'delete' => 1,
          'resize' => 1,
          'mkdir' => 1,
          'rmdir' => 1,
        ),
      ),
      'thumbnails' => array(),
      'mkdirnum' => '0',
    ),
    2 => array(
      'name' => 'admin profile',
      'usertab' => 1,
      'filesize' => '0',
      'quota' => '0',
      'tuquota' => '0',
      'extensions' => '*',
      'dimensions' => '1200x1200',
      'filenum' => '0',
      'directories' => array(
        0 => array(
          'name' => 'content',
          'subnav' => 1,
          'browse' => 1,
          'upload' => 1,
          'thumb' => 1,
          'delete' => 1,
          'resize' => 1,
          'mkdir' => 1,
          'rmdir' => 1,
        ),
      ),
      'thumbnails' => array(),
      'mkdirnum' => '0',
    ),
    3 => array(
      'name' => 'editor profile',
      'usertab' => 1,
      'filesize' => '0',
      'quota' => '0',
      'tuquota' => '0',
      'extensions' => '*',
      'dimensions' => '1200x1200',
      'filenum' => '0',
      'directories' => array(
        0 => array(
          'name' => 'content',
          'subnav' => 1,
          'browse' => 1,
          'upload' => 1,
          'thumb' => 1,
          'delete' => 0,
          'resize' => 1,
          'mkdir' => 1,
          'rmdir' => 1,
        ),
      ),
      'thumbnails' => array(),
      'mkdirnum' => '0',
    ),
  );
  variable_set('imce_profiles', $imce_profiles);

  $imce_roles_profiles = array(
    4 => array(
      'weight' => '-10',
      'pid' => '2',
    ),
    3 => array(
      'weight' => '-9',
      'pid' => '3',
    ),
    2 => array(
      'weight' => 11,
      'pid' => '0',
    ),
    1 => array(
      'weight' => 12,
      'pid' => '0',
    ),
  );
  variable_set('imce_roles_profiles', $imce_roles_profiles);

  _p6_profile_log(st('Configured other variables.'));
}

/**
 * Installation task to set up Backup & Migrate module.
 *
 * Define backup destinations, schedules and profile.
 */
function _p6_profile_set_backupmigrate() {
  require_once './' . drupal_get_path('module', 'backup_migrate') . '/backup_migrate.module';
  require_once './' . drupal_get_path('module', 'backup_migrate') . '/includes/crud.inc';
  require_once './' . drupal_get_path('module', 'backup_migrate') . '/includes/profiles.inc';
  require_once './' . drupal_get_path('module', 'backup_migrate') . '/includes/files.inc';
  require_once './' . drupal_get_path('module', 'backup_migrate') . '/includes/schedules.inc';

  // Destinations.
  $dir = file_directory_path() . '/backup_migrate/';

  // Hourly Backups Directory.
  $destination_hourly = array(
    'destination_id' => 'hourly_backups',
    'location' => $dir . 'scheduled/hourly',
    'name' => t('Hourly Backups Directory'),
    'type' => 'file',
  );
  $destination_hourly = backup_migrate_crud_create_item('destination', $destination_hourly);
  $destination_hourly->save();

  // Daily Backups Directory.
  $destination_daily = array(
    'destination_id' => 'daily_backups',
    'location' => $dir . 'scheduled/daily',
    'name' => t('Daily Backups Directory'),
    'type' => 'file',
  );
  $destination_daily = backup_migrate_crud_create_item('destination', $destination_daily);
  $destination_daily->save();

  // Weekly Backups Directory.
  $destination_weekly = array(
    'destination_id' => 'weekly_backups',
    'location' => $dir . 'scheduled/weekly',
    'name' => t('Weekly Backups Directory'),
    'type' => 'file',
  );
  $destination_weekly = backup_migrate_crud_create_item('destination', $destination_weekly);
  $destination_weekly->save();

  $nodata_tables = array(
    'cache',
    'cache_filter',
    'cache_calendar_ical',
    'cache_menu',
    'cache_page',
    'cache_views',
    'cache_block',
    'cache_update',
    'cache_form',
    'sessions',
    'search_dataset',
    'search_index',
    'search_keywords_log',
    'search_total',
    'watchdog',
    'accesslog',
    'devel_queries',
    'devel_times',
  );

  $settings = array(
    'profile_id' => 'project6',
    'filename' => variable_get("backup_migrate_file_name", 'backup'),
    'append_timestamp' => 1,
    'timestamp_format' => 'Y-m-d\TH-i-s',
    'filters' => array(
      'compression' => (@function_exists("gzencode")) ? 'gzip' : 'none',
      'exclude_tables' => variable_get("backup_migrate_exclude_tables", array()),
      'nodata_tables' => $nodata_tables,
    ),
    'name' => t('Project6 Settings'),
  );

  $profile_project6 = backup_migrate_crud_create_item('profile', $settings);
  $profile_project6->save();


  // Hourly schedule.
  $schedule_hourly = array(
    'name' => t('Hourly Schedule'),
    'profile_id' => 'project6',
    'enabled' => 1,
    'destination_id' => 'hourly_backups',
    'period' => 3600,
    'keep' => 48,
  );
  $schedule_hourly = backup_migrate_crud_create_item('schedule', $schedule_hourly);
  $schedule_hourly->save();

  // Daily schedule.
  $schedule_daily = array(
    'name' => t('Daily Schedule'),
    'profile_id' => 'project6',
    'enabled' => 1,
    'destination_id' => 'daily_backups',
    'period' => 86400,
    'keep' => 14,
  );
  $schedule_daily = backup_migrate_crud_create_item('schedule', $schedule_daily);
  $schedule_daily->save();

  // Weekly schedule.
  $schedule_weekly = array(
    'name' => t('Weekly Schedule'),
    'profile_id' => 'project6',
    'enabled' => 1,
    'destination_id' => 'weekly_backups',
    'period' => 604800,
    'keep' => 8,
  );
  $schedule_weekly = backup_migrate_crud_create_item('schedule', $schedule_weekly);
  $schedule_weekly->save();

  _p6_profile_log(st('Configured Backup&Migrate module.'));
}

/**
 * Installation task for cleaning up.
 */
function _p6_cleanup() {
  drupal_rebuild_theme_registry();
  menu_rebuild();
  node_types_rebuild();
  install_init_blocks();
  views_invalidate_cache();
}

/**
 * Implements hook_form_FORM_ID_alter().
 *
 * Set P6 profile as the default install profile.
 */
function system_form_install_select_profile_form_alter(&$form, $form_state) {
  foreach ($form['profile'] as $key => $element) {
    $form['profile'][$key]['#value'] = 'p6_profile';
  }
}

/**
 * Implements hook_form_alter().
 */
function p6_profile_form_alter(&$form, $form_state, $form_id) {
  if ($form_id == 'install_configure') {
    // Set default for site name field.
    $form['site_information']['site_name']['#default_value'] = $_SERVER['SERVER_NAME'];
    $form['admin_account']['account']['name']['#default_value'] = ADMINISTRATOR_USERNAME;
  }
}

/**
 * Generates a random alphanumeric password.
 */
function _p6_profile_password($length = 15) {
  // This variable contains the list of allowable characters for the
  // password. Note that the number 0 and the letter 'O' have been
  // removed to avoid confusion between the two. The same is true
  // of 'I', 1, and 'l'.
  $allowable_characters = 'abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ23456789!@#$%^&*()?';

  // Zero-based count of characters in the allowable list:
  $len = strlen($allowable_characters) - 1;

  // Declare the password as a blank string.
  $pass = '';

  // Loop the number of times specified by $length.
  for ($i = 0; $i < $length; $i++) {

    // Each iteration, pick a random character from the
    // allowable string and append it to the password:
    $pass .= $allowable_characters[mt_rand(0, $len)];
  }

  return $pass;
}

/**
 * Consolidate logging.
 */
function _p6_profile_log($msg) {
  error_log($msg);
  drupal_set_message($msg);
}
