<?php
/**
 * @file
 *
 * @author Osman Gormus <osman@project6.com>
 *
 * 2012-01-10
 */

require_once 'theme-settings.php';

require_once 'theme-functions.php';

/**
 * Initialize theme settings
 */
global $theme_key;

if (db_is_active()) {
  p6_base_initialize_theme_settings($theme_key);
}


/**
 * Implements template_preprocess.
 */
function p6_base_preprocess(&$vars, $hook) {
  global $user;

  // Create global variable(s) for the logged-in user's role name(s) as
  // $vars['is_role-name'].
  $user_roles = $user->roles;
  $all_roles  = user_roles();

  unset($user_roles[1]);
  unset($user_roles[2]);

  foreach (array_intersect($all_roles, $user_roles) as $role_name) {
    $vars['is_' . _p6_base_css_safe($role_name)] = TRUE;
  }

  $vars['logged_in'] = ($user->uid > 0) ? TRUE : FALSE;
  $vars['ui_admin_links'] = array();

  static $ui_count = array();
  $ui_count[$hook] = isset($ui_count[$hook]) && is_int($ui_count[$hook]) ? $ui_count[$hook] : 1;
  $vars['ui_zebra'] = ($ui_count[$hook] % 2) ? 'ui-odd ui-' . $hook . '-odd' : 'ui-even ui-' . $hook . '-even';
  $vars['ui_id'] = $ui_count[$hook]++;
}


/**
 * Implements template_preprocess_page.
 */
function p6_base_preprocess_page(&$vars) {
  global $language, $theme_key, $theme_info, $theme_path, $user;

  // Path to the base theme.
  $vars['base_theme_path'] = drupal_get_path('theme', 'p6_base');

  // Path to the active theme.
  $vars['active_theme_path'] = drupal_get_path('theme', $theme_key);

  // Adds AddThis namespace to HTML.
  $vars['addthis_xmlns'] = (theme_get_setting('addthis_namespace') == 1) ? 'xmlns:addthis="http://www.addthis.com/help/api-spec"' : '';

  // Adds Facebook namespace to HTML.
  $vars['facebook_xmlns'] = (theme_get_setting('facebook_namespace') == 1) ? 'xmlns:fb="http://www.facebook.com/2008/fbml"' : '';

  $vars['no_javascript'] = (theme_get_setting('display_enable_js_msg') == 1) ? '<div id="ui-nojs"><div class="ui-inner ui-page-width clearfix"><a rel="nofollow" href="http://www.enable-javascript.com/">Enable JavaScript</a></div></div>' : '';

  // Remove sidebars if disabled e.g., for Panels.
  if (!$vars['show_blocks']) {
    $vars['sidebar_first'] = '';
    $vars['sidebar_last'] = '';
  }

  $vars['search_box'] = (theme_get_setting('toggle_search') ? '<div class="ui-block-search clearfix"><div>' . drupal_get_form('search_theme_form') . '</div></div>' : '');

  // Build array of helpful body classes.
  $body_classes = array();
  $body_classes[] = ($vars['logged_in']) ? 'logged-in' : 'not-logged-in';

  // Add logged-in user's role name(s) to body classes array as "is_role-name".
  $user_roles = $user->roles;
  $all_roles  = user_roles();

  unset($user_roles[1]);
  unset($user_roles[2]);

  foreach (array_intersect($all_roles, $user_roles) as $role_name) {
    $body_classes[] = 'is_' . _p6_base_css_safe($role_name);
  }

  $body_classes[] = ($vars['is_front']) ? 'front' : 'not-front';
  $body_classes[] = ($vars['messages']) ? 'with-messages' : 'not-with-messages';
  $body_classes[] = ($vars['help']) ? 'with-help' : 'not-with-help';

  if (isset($vars['node'])) {
    $body_classes[] = ($vars['node']) ? 'full-node' : '';
    $body_classes[] = ($vars['node']->type) ? 'node-type-' . $vars['node']->type : '';
    $body_classes[] = ($vars['node']) ? 'nid-' . $vars['node']->nid : '';
  }

  if ($vars['sidebar_first']) {
    $body_classes[] = 'ui-sidebar-first';
  }

  if ($vars['sidebar_last']) {
    $body_classes[] = 'ui-sidebar-last';
  }

  // Path, Level & Section logic:
  // ==
  // i.e. http://example.com/
  // level-1
  // --
  // i.e. http://example.com/somepage
  // level=2, path-somepage, section-somepage
  // --
  // i.e. http://example.com/some_page/anotherpage
  // level=3, path-somepage_anotherpage, section-somepage
  $path = drupal_get_path_alias($_GET['q']);
  if ($path) {
    if (!$vars['is_front']) {
      $level = explode('/', $path);
      $path_request = explode('/', request_uri());

      // Level of depth.
      $body_classes[] = 'level-' . (string) (count($level) + 1);
      // Full path.
      $body_classes[] = _p6_base_css_safe('path-' . $path);
      // Last part of the path.
      $body_classes[] = _p6_base_css_safe('pathlast-' . end($path_request));
      // First part of the path as a section name.
      $body_classes[] = _p6_base_css_safe('section-' . $path_request['1']);
    }
  }

  // Remove empty elements.
  $body_classes = array_filter($body_classes);

  // Create class list separated by spaces.
  $vars['body_classes'] = implode(' ', $body_classes);

  // Add a unique page id.
  $vars['body_id'] = 'pid-' . (($vars['is_front']) ? 'front' : _p6_base_css_safe($path));


  // Load condition based CSS files.
  if (menu_local_tasks(0, FALSE)) {
    drupal_add_css(drupal_get_path('theme', 'p6_base') . '/css/ui-tabs.css', 'theme', 'all');
  }
  // Rebuild $styles.
  $css = drupal_add_css();
  $vars['styles'] = drupal_get_css($css);
}

/**
 * Process variables for node.tpl.php
 *
 * @see node.tpl.php
 */
function p6_base_preprocess_node(&$vars) {
  global $user;
  $node = $vars['node'];

  $vars['links'] = !empty($node->links) ? theme('links', $node->links, array('class' => 'ui-links clearfix')) : '';

  $node_classes[] = 'ui-node';
  $node_classes[] = 'ui-node-' . $node->type;

  $node_classes[] = (($node->language) ? 'ui-lang-' . $node->language : 'ui-lang-neutral');
  $node_classes[] = (($node->status == 1) ? '' : 'ui-node-not-published');
  $node_classes[] = (($node->promote == 1) ? 'ui-node-promoted' : '');
  $node_classes[] = (($node->sticky == 1) ? 'ui-node-sticky' : '');
  $node_classes[] = (($node->teaser) ? 'ui-node-teaser' : '');

  $node_classes[] = $vars['ui_zebra'];
  $node_classes[] = 'ui-node-' . $vars['ui_id'];

  if (($node->teaser) && node_access('update', $node)) {
    $ui_admin_links[] = l(t('Edit'), 'node/' . $node->nid . '/edit',
        array(
          'attributes' => array(
            'title' => t('Edit this node'),
            'class' => 'ui-links-edit',
          ),
          'query' => drupal_get_destination(),
        )
    );
  }
  if (!empty($ui_admin_links)) {
    drupal_add_css(drupal_get_path('theme', 'p6_base') . '/css/ui-admin-links.css', 'theme', 'all');
    $vars['ui_admin_links'] = '<div class="ui-admin-links">' . implode(' ', $ui_admin_links) . '</div>';
    $node_classes[] = 'ui-admin-links-active';
  }

  $node_classes[] = 'clearfix';
  $node_classes = array_filter($node_classes);
  $vars['ui_node_classes'] = implode(' ', $node_classes);
  $vars['ui_node_id'] = 'ui-node-' . $node->nid;
}

/**
 * Format the "Submitted by username on date/time" for each node
 */
function p6_base_node_submitted($node) {
  $node_submitted = t('Submitted by !username on @datetime',
    array(
      '!username' => theme('username', $node),
      '@datetime' => format_date($node->created),
    )
  );

  return '<div class="ui-node-submitted">' . $node_submitted . '</div>';
}

/**
 * Process variables for block.tpl.php
 *
 * @see block.tpl.php
 */
function p6_base_preprocess_block(&$vars) {
  $block = $vars['block'];

  static $block_counter = array();
  // All blocks get an independent counter for each region.
  if (!isset($block_counter[$block->region])) {
    $block_counter[$block->region] = 1;
  }
  $vars['block_id'] = $block_counter[$block->region]++;

  if ($block->module == 'nodeasblock') {
    $vars['title'] = strip_tags($block->subject);
  }
  else {
    $vars['title'] = $block->subject;
  }

  $vars['content'] = $block->content;

  // Classes.
  $block_classes[] = 'ui-block';
  $block_classes[] = 'ui-block-' . $block->module;
  if ($vars['block_id'] == 1) {
    $block_classes[] = 'ui-block-first';
  }
  if ($vars['block_id'] == count(block_list($block->region))) {
    $block_classes[] = 'ui-block-last';
  }
  $block_classes[] = $vars['ui_zebra'];
  $block_classes[] = 'ui-block-' . $vars['block_id'];

  if ($block->module == 'nodeasblock') {
    // Add content-type that nodeasblock is used.
    $node = node_load($block->delta);
    $block_classes[] = 'ui-block-nodeasblock-' . $node->type;

    // Add template suggestions.
    $vars['template_files'][] = 'block-' . $node->type;
    $vars['template_files'][] = 'block-' . $node->type . '-' . $block->delta;
  }

  // Configure links.
  if (user_access('administer blocks')) {
    if ($block->module == 'block') {
      $ui_admin_links[] = l(t('Configure'), 'admin/build/block/configure/' . $block->module . '/' . $block->delta,
        array(
          'attributes' => array(
            'title' => t('Configure this block'),
            'class' => 'ui-block-links-block-config',
          ),
          'query' => drupal_get_destination(),
        )
      );
    }
    // For node blocks.
    elseif ($block->module == 'nodeasblock') {
      if (node_access('update', node_load($block->delta))) {
        $ui_admin_links[] = l(t('Edit'), 'node/' . $block->delta . '/edit',
          array(
            'attributes' => array(
              'title' => t('Edit this node'),
              'class' => 'ui-block-links-node-edit',
            ),
            'query' => drupal_get_destination(),
            'fragment' => 'nodeasblockset',
          )
        );
      }
    }
    // For other blocks.
    else {
      $ui_admin_links[] = l(t('Configure'), 'admin/build/block/configure/' . $block->module . '/' . $block->delta,
        array(
          'attributes' => array(
            'title' => t('Configure this @module block', array('@module' => $block->module)),
            'class' => 'ui-block-links-block-config',
          ),
          'query' => drupal_get_destination(),
        )
      );
    }
  }
  // Display 'edit view' for Views blocks.
  if ($block->module == 'views' && user_access('administer views')) {
    list($view_name, $view_block) = explode('-block', $block->delta);
    $ui_admin_links[] = l(t('Edit'), 'admin/build/views/edit/' . $view_name,
      array(
        'attributes' => array(
          'title' => t("Edit '@view' view", array('@view' => $view_name)),
          'class' => 'ui-block-links-view-edit',
        ),
        'query' => drupal_get_destination(),
        'fragment' => 'views-tab-block' . $view_block,
      )
    );
    // Export view.
    $ui_admin_links[] = l(t('Export'), 'admin/build/views/export/' . $view_name,
      array(
        'attributes' => array(
          'title' => t("Export '@view' view", array('@view' => $view_name)),
          'class' => 'ui-block-links-view-export',
        ),
        'query' => drupal_get_destination(),
      )
    );
    // Clone view.
    $ui_admin_links[] = l(t('Clone'), 'admin/build/views/clone/' . $view_name,
      array(
        'attributes' => array(
          'title' => t("Clone '@view' view", array('@view' => $view_name)),
          'class' => 'ui-block-links-view-clone',
        ),
        'query' => drupal_get_destination(),
      )
    );
  }
  // Display 'edit menu' for Menu blocks.
  elseif (($block->module == 'menu' || ($block->module == 'user' && $block->delta == 1)) && user_access('administer menu')) {
    $menu_name = ($block->module == 'user') ? 'navigation' : $block->delta;
    $ui_admin_links[] = l(t('Edit menu'), 'admin/build/menu-customize/' . $menu_name,
      array(
        'attributes' => array(
          'title' => t('Edit @menu menu', array('@menu' => $menu_name)),
          'class' => 'ui-block-links-menu-edit',
        ),
        'query' => drupal_get_destination(),
      )
    );
  }
  // Display 'edit menu' for Menu block blocks.
  elseif ($block->module == 'menu_block' && user_access('administer menu')) {
    list($menu_name, ) = split(':', variable_get("menu_block_{$block->delta}_parent", 'navigation:0'));
    $block_links[] = l(t('Edit menu XX'), 'admin/build/menu-customize/' . $menu_name,
      array(
        'attributes' => array(
          'title' => t('Edit @menu menu', array('@menu' => $menu_name)),
          'class' => 'ui-block-links-menu-edit',
        ),
        'query' => drupal_get_destination(),
      )
    );
  }

  if (!empty($ui_admin_links)) {
    drupal_add_css(drupal_get_path('theme', 'p6_base') . '/css/ui-admin-links.css', 'theme', 'all');
    $vars['ui_admin_links'] = '<div class="ui-admin-links">' . implode(' ', $ui_admin_links) . '</div>';
    $block_classes[] = 'ui-admin-links-active';
  }


  $block_classes[] = 'clearfix';
  $block_classes = array_filter($block_classes);
  $vars['ui_block_classes'] = implode(' ', $block_classes);

  $vars['ui_block_id'] = 'ui-block-' . $block->module . '-' . $block->delta;
}

/**
 * Process variables for nodeasblock.tpl.php.
 *
 * The $variables array contains the following arguments:
 * - $node
 *
 * @see nodeasblock.tpl.php
 */
function p6_base_preprocess_nodeasblock(&$vars) {
  $node = $vars['node'];
  $node = node_load($node->nid, NULL, TRUE);
  $node->build_mode = NODE_BUILD_NORMAL;
  $node = node_build_content($node, FALSE, TRUE);

  // If the user has permissions to edit the node, then add a link.
  if (node_access('update', $node)) {
    $vars['edit_link'] = l(t('[edit]'), 'node/' . $node->nid . '/edit', array("title" => t("Edit")));
  }
  $vars['content'] = drupal_render($node->content);

  $vars['template_files'][] = 'nodeasblock-' . $vars['node']->type;
  $vars['template_files'][] = 'nodeasblock-' . $vars['node']->type . '-' . $vars['node']->nid;
}

/**
 * Format a link to the more extensive filter tips.
 *
 * @ingroup themeable
 */
function p6_base_filter_tips_more_info() {
  return '<div class="ui-filter-tips-more-info">' . l(t('More information about formatting options'), 'filter/tips') . '</div>';
}

/**
 * Theme function for fitler tips more info.
 *
 * This is copied from theme_fitler_tips_more_info() with small modifications.
 *
 * @return string
 *   Filter tips more info HTML.
 */
function p6_base_better_formats_filter_tips_more_info() {
  $text = variable_get('better_formats_long_tips_link_text', '');
  if (module_exists('i18n_strings') && $text) {
    $text = tt($text);
  }
  else {
    $text = $text ? $text : t('More information about formatting options');
  }
  return '<div class="ui-filter-tips-more-info">' . l($text, 'filter/tips') . '</div>';
}

/**
 * Returns a themed breadcrumb trail.
 *
 * @param array $breadcrumb
 *   An array containing the breadcrumb links.
 *
 * @return string
 *   A string containing the breadcrumb output.
 */
function p6_base_breadcrumb($breadcrumb) {
  if (!empty($breadcrumb)) {
    return '<div class="ui-breadcrumb clearfix">' . implode(' » ', $breadcrumb) . '</div>';
  }
}

/**
 * Return a themed help message.
 *
 * @return string
 *   A string containing the helptext for the current page.
 */
function p6_base_help() {
  if ($help = menu_get_active_help()) {
    drupal_add_css(drupal_get_path('theme', 'p6_base') . '/css/ui-info.css', 'theme', 'all');

    return '<div class="ui-help clearfix" title="' . t('Help') . '">' . $help . '</div>';
  }
}

/**
 * Return a themed set of status and/or error messages. The messages are grouped
 * by type.
 *
 * @param $display
 *   Set to 'status' or 'error' to display only messages of that type.
 *
 * @return
 *   A string containing the messages.
 */
function p6_base_status_messages($display = NULL) {
  $output = '';

  foreach (drupal_get_messages($display) as $type => $messages) {
    $output .= '<div class="ui-' . $type . ' clearfix" title="' . t(drupal_ucfirst($type)) . '">';
    $output .= '<h4 class="ui-title">' . t(drupal_ucfirst($type)) . '</h4>';
    if (count($messages) > 1) {
      $output .= '<ul class="ui-messages">';
      foreach ($messages as $message) {
        $output .= '<li class="ui-message">' . $message . '</li>';
      }
      $output .= '</ul>';
    }
    else {
      $output .= '<div class="ui-message">' . $messages[0] . '</div>';
    }
    $output .= '</div>';
  }

  if ($messages) {
    drupal_add_css(drupal_get_path('theme', 'p6_base') . '/css/ui-info.css', 'theme', 'all');

    return $output;
  }
  else {
    return '';
  }
}

/**
 * Implements theme_menu_local_tasks().
 */
function p6_base_menu_local_tasks() {
  $output = '';

  if ($primary = menu_primary_local_tasks()) {
    $output .= '<ul class="ui-tabs-primary clearfix">' . $primary . '</ul>';
  }
  if ($secondary = menu_secondary_local_tasks()) {
    $output .= '<ul class="ui-tabs-secondary clearfix">' . $secondary . '</ul>';
  }
  if ($output != '') {
    $output = '<div id="ui-tabs" class="clearfix">' . $output . '</div>';
  }

  return $output;
}

/**
 * Implements theme_menu_local_task().
 */
function p6_base_menu_local_task($link, $active = FALSE) {
  return '<li' . ($active ? ' class="ui-tab-active"' : '') . '>' . $link . '</li>';
}

/**
 * Implements theme_search_theme_form()
 */
function p6_base_search_theme_form($form) {
  $form['search_theme_form']['#title'] = t('Search this site');
  $form['search_theme_form']['#default_value'] = '';
  $form['search_theme_form']['#attributes']['title'] = t('Enter the terms you wish to search for.');
  $form['search_theme_form']['#description'] = t('Enter the terms you wish to search for.');

  $form['submit']['#value'] = t('Search');

  return drupal_render($form);
}

/**
 * Implements theme_search_block_form()
 */
function p6_base_search_block_form($form) {
  $form['search_block_form']['#title'] = t('Search this site');
  $form['search_block_form']['#default_value'] = '';
  $form['search_block_form']['#attributes']['title'] = t('Enter the terms you wish to search for.');
  $form['search_block_form']['#description'] = t('Enter the terms you wish to search for.');

  $form['submit']['#value'] = t('Search');

  return drupal_render($form);
}

/**
 * Returns code that emits the 'more' link used on blocks.
 *
 * @param string $url
 *   The url of the main page
 *
 * @param string $title
 *   A descriptive verb for the link, like 'Read more'
 */
function p6_base_more_link($url, $title) {
  return '<div class="more-link">' . t('<a class="more" href="@link" title="@title">more</a>', array('@link' => check_url($url), '@title' => $title)) . '</div>';
}

/**
 * Converts a string to a suitable html ID attribute.
 *
 * http://www.w3.org/TR/html4/struct/global.html#h-7.5.2 specifies what makes a
 * valid ID attribute in HTML. This function:
 *
 * - Ensure an ID starts with an alpha character by optionally adding an 'n'.
 * - Replaces any character except A-Z, numbers, and underscores with dashes.
 * - Converts entire string to lowercase.
 *
 * @param string $string
 *   The string
 *
 * @return string
 *   The converted string
 */
function _p6_base_css_safe($string) {
  $string = strtolower(preg_replace('/[^a-zA-Z0-9_-]+/', '-', $string));
  return $string;
}
