<?php
/**
 * @file
 *
 * Default theme settings.
 */
function default_theme_settings() {
  $defaults = array(
    'rebuild_registry'                      => 0,
    'addthis_namespace'                     => 0,
    'facebook_namespace'                    => 0,
    'disable_site_slogan'                   => 1,
    'disable_site_mission'                  => 1,
    'disable_site_footer'                   => 1,
    'display_enable_js_msg'                 => 1,
  );

  // Merge global theme settings with theme settings.
  $defaults = array_merge($defaults, theme_get_settings());

  return $defaults;
}

/**
 * Initialize theme settings if needed.
 */
function p6_base_initialize_theme_settings($theme_name) {
  $theme_settings = theme_get_settings($theme_name);
  if (!isset($theme_settings['primary_menu_dropdown']) || $theme_settings['rebuild_registry'] == 1) {
    static $registry_rebuilt = FALSE;

    // Rebuild theme registry & notify user.
    if (isset($theme_settings['rebuild_registry']) && $theme_settings['rebuild_registry'] == 1 && !$registry_rebuilt) {
      drupal_rebuild_theme_registry();
      drupal_set_message(t('Theme registry rebuild completed. <a href="!link">Turn off</a> this feature for production websites.', array('!link' => url('admin/build/themes/settings/' . check_plain($GLOBALS['theme'])))), 'warning');
      $registry_rebuilt = TRUE;
    }

    // Retrieve saved or site-wide theme settings.
    $theme_setting_name = str_replace('/', '_', 'theme_'. $theme_name .'_settings');
    $settings = (variable_get($theme_setting_name, FALSE)) ? theme_get_settings($theme_name) : theme_get_settings();

    // Skip toggle_node_info_ settings.
    if (module_exists('node')) {
      foreach (node_get_types() as $type => $name) {
        unset($settings['toggle_node_info_' . $type]);
      }
    }

    // Combine default theme settings from .info file & theme-settings.php
    $theme_data = list_themes();
    $info_theme_settings = ($theme_name && isset($theme_data[$theme_name]->info['settings'])) ? $theme_data[$theme_name]->info['settings'] : array();
    $defaults = array_merge(default_theme_settings(), $info_theme_settings);

    // Set combined default & saved theme settings.
    variable_set($theme_setting_name, array_merge($defaults, $settings));

    // Force theme settings refresh.
    theme_get_setting('', TRUE);
  }
}

function phptemplate_settings($saved_settings) {
  global $base_url;

  // Get theme name from url (admin/.../theme_name)
  $theme_name = arg(count(arg()) - 1);

  // Combine default theme settings from .info file & theme-settings.php
  $theme_data = list_themes();
  $info_theme_settings = ($theme_name && isset($theme_data[$theme_name]->info['settings'])) ? $theme_data[$theme_name]->info['settings'] : array();
  $defaults = array_merge(default_theme_settings(), $info_theme_settings);

  // Combine default and saved theme settings.
  $settings = array_merge($defaults, $saved_settings);

  // Create theme settings form widgets using Forms API.
  $form['p6_base_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Project6 Theme settings'),
    '#description' => '',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['p6_base_settings']['addthis_namespace'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add AddThis namespace'),
    '#default_value' => $settings['addthis_namespace'],
    '#description' => t('Add namespace for addthis'),
  );

  $form['p6_base_settings']['facebook_namespace'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add Facebook namespace'),
    '#default_value' => $settings['facebook_namespace'],
    '#description' => t('Add namespace for Facebook'),
  );

  $form['p6_base_settings']['rebuild_registry'] = array(
    '#type' => 'checkbox',
    '#title' => t('Rebuild theme registry'),
    '#default_value' => $settings['rebuild_registry'],
    '#description' => t('This setting is useful while developing themes (see <a href="!link">rebuilding the theme registry</a>). However, it <strong>significantly degrades performance</strong> and should be turned off for any production website.', array('!link' => 'http://drupal.org/node/173880#theme-registry')),
  );

  $form['p6_base_settings']['display_enable_js_msg'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable JavaScript warning'),
    '#default_value' => $settings['display_enable_js_msg'],
    '#description' => t("If Javascript is disabled in visitor's web browser, following message will be displayed to the visitor:") . ' ' . l(t('For full functionality of this site it is necessary to enable JavaScript. Here are the instructions how to enable JavaScript in your web browser.'), 'http://www.enable-javascript.com/', array('attributes' => array('rel' => 'nofollow'))),
  );

  if (module_exists('p6_helper')) {
    $form['p6_base_settings']['drupal_features'] = array(
      '#type' => 'fieldset',
      '#title' => t('Drupal core features'),
      '#description' => t('Overwrites "Toggle Display" settings above. Any content previously defined at <a href="!link">Site Information</a> will be erased.', array('!link' => url('admin/settings/site-information'))),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    $form['p6_base_settings']['drupal_features']['disable_site_slogan'] = array(
      '#type' => 'checkbox',
      '#title' => t('Disable site slogan'),
      '#default_value' => $settings['disable_site_slogan'],
    );

    $form['p6_base_settings']['drupal_features']['disable_site_mission'] = array(
      '#type' => 'checkbox',
      '#title' => t('Disable site mission'),
      '#default_value' => $settings['disable_site_mission'],
    );

    $form['p6_base_settings']['drupal_features']['disable_site_footer'] = array(
      '#type' => 'checkbox',
      '#title' => t('Disable site footer'),
      '#default_value' => $settings['disable_site_footer'],
    );
  }

  // Return theme settings form.
  return $form;

}
